Imports System.IO
Imports System.Drawing.Imaging

Public Class Form1
    Private Const Wid As Integer = 800
    Private Const Hgt As Integer = 800
    Private Const XOff As Integer = 150
    Private Const YOff As Integer = 200
    Private Hits(Wid, Hgt) As Integer
    Private CurrentPoint As Point
    Private CurrentPrime As Long

    ' Initialize data structures.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        CurrentPoint = New Point(0, 0)
        CurrentPrime = 1
    End Sub

    ' Stop running.
    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        Running = False
    End Sub

    ' Start or stop.
    Private Running As Boolean = False
    Private Sub mnuFileStart_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFileStart.Click
        If (mnuFileStart.Text = "&Start") Then
            ' Start.
            Running = True
            mnuFileStart.Text = "&Stop"

            DrawFractal()

            ' Clean up after we finish running.
            mnuFileStart.Enabled = True
            mnuFileStart.Text = "&Start"
        Else
            ' Stop.
            Running = False
            mnuFileStart.Enabled = False
        End If
    End Sub

    ' Add points to the fractal.
    Private NumPoints As Integer = 0
    Private Sub DrawFractal()
        Const points_per_loop As Integer = 10000
        Do While (Running)
            ' Generate a bunch of points.
            For i As Integer = 1 To points_per_loop
                ' Find the next prime.
                CurrentPrime = FindNextPrime(CurrentPrime)

                ' See which kind of prime it is.
                Select Case (CurrentPrime Mod 5)
                    Case 1
                        CurrentPoint.Y -= 1
                    Case 2
                        CurrentPoint.X += 1
                    Case 3
                        CurrentPoint.Y += 1
                    Case 4
                        CurrentPoint.X -= 1
                End Select

                ' Record the hit.
                Dim ix As Integer = CurrentPoint.X + XOff
                Dim iy As Integer = CurrentPoint.Y + YOff
                If (ix >= 0 AndAlso iy >= 0 AndAlso ix < Wid AndAlso iy < Hgt) Then
                    Hits(ix, iy) += 1
                End If
            Next i

            ' Build the image.
            BuildImage()

            ' Display the point count.
            NumPoints += points_per_loop
            lblNumPoints.Text = NumPoints.ToString()
            lblPrime.Text = CurrentPrime.ToString()

            ' Process button clicks if there were any.
            Application.DoEvents()
        Loop
    End Sub

    ' Find the next prime after this one.
    Private Function FindNextPrime(ByVal value As Long) As Long
        ' Cheat a little for speed.
        'if (value = 1) return 2
        'if (value = 2) return 3
        Dim i As Long = value + 2
        Do
            If (IsPrime(i)) Then Return i
            i += 2
        Loop
    End Function

    ' Return true of the value is prime.
    ' For speed, asssume value > 2 and value is odd.
    Private Function IsPrime(ByVal value As Long) As Boolean
        Dim stop_at As Long = Int(Math.Sqrt(value))
        For factor As Long = 3 To stop_at Step 2
            If (value Mod factor = 0) Then Return False
        Next factor
        Return True
    End Function

    ' Build and display the image.
    Private Sub BuildImage()
        Dim bm As Bitmap = New Bitmap(Wid, Hgt)
        Using gr As Graphics = Graphics.FromImage(bm)
            ' Clear.
            gr.Clear(Color.Black)

            ' Find the largest hit value.
            Dim max As Integer = 0
            For x As Integer = 0 To Wid - 1
                For y As Integer = 0 To Hgt - 1
                    If (max < Hits(x, y)) Then max = Hits(x, y)
                Next y
            Next x

            ' Plot the hits.
            Dim fmax As Single = max
            For x As Integer = 0 To Wid - 1
                For y As Integer = 0 To Hgt - 1
                    If (Hits(x, y) > 0) Then
                        bm.SetPixel(x, y, MapRainbowColor(Hits(x, y), 1, fmax))
                    End If
                Next y
            Next x

            ' Draw the axes.
            gr.DrawLine(Pens.Blue, XOff, 0, XOff, Hgt)
            gr.DrawLine(Pens.Blue, 0, YOff, Wid, YOff)
        End Using

        ' Display the result.
        picFractal.Image = bm
    End Sub

    ' Save the file.
    Private Sub mnuFileSave_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFileSave.Click
        If (sfdFractal.ShowDialog() = DialogResult.OK) Then
            Dim bm As Bitmap = DirectCast(picFractal.Image, Bitmap)
            SaveBitmapUsingExtension(bm, sfdFractal.FileName)
        End If
    End Sub

    ' Save the file with the appropriate format.
    ' Throw a NotSupportedException if the file
    ' has an unknown extension.
    Public Sub SaveBitmapUsingExtension(ByVal bm As Bitmap, ByVal filename As String)
        Dim extension As String = Path.GetExtension(filename)
        Select Case (extension.ToLower())
            Case ".bmp"
                bm.Save(filename, ImageFormat.Bmp)
            Case ".exif"
                bm.Save(filename, ImageFormat.Exif)
            Case ".gif"
                bm.Save(filename, ImageFormat.Gif)
            Case ".jpg", ".jpeg"
                bm.Save(filename, ImageFormat.Jpeg)
            Case ".png"
                bm.Save(filename, ImageFormat.Png)
            Case ".tif", ".tiff"
                bm.Save(filename, ImageFormat.Tiff)
            Case Else
                Throw New NotSupportedException( _
                    "Unknown file extension " & extension)
        End Select
    End Sub

    ' Map a value to a rainbow color.
    Private Function MapRainbowColor(ByVal value As Single, ByVal red_value As Single, ByVal blue_value As Single) As Color
        ' Convert into a value between 0 and 1023.
        Dim int_value As Integer = Int(1023 * (value - red_value) / (blue_value - red_value))

        ' Map different color bands.
        If (int_value < 256) Then
            ' Red to yellow. (255, 0, 0) to (255, 255, 0).
            Return Color.FromArgb(255, int_value, 0)
        ElseIf (int_value < 512) Then
            ' Yellow to green. (255, 255, 0) to (0, 255, 0).
            int_value -= 256
            Return Color.FromArgb(255 - int_value, 255, 0)
        ElseIf (int_value < 768) Then
            ' Green to aqua. (0, 255, 0) to (0, 255, 255).
            int_value -= 512
            Return Color.FromArgb(0, 255, int_value)
        Else
            ' Aqua to blue. (0, 255, 255) to (0, 0, 255).
            int_value -= 768
            Return Color.FromArgb(0, 255 - int_value, 255)
        End If
    End Function
End Class
